"use strict";
var SignatureClient = require('./SignatureClient'),
    CommonUtil = require('./Util'),
    config = require('./config');

let PwZSDK = function (merchantID, privateKey, publicKey, signatoryJID, zetaPublicKey, successURL, failureURL, notifyURL) {
    for (let value of arguments) {
        if (value === null || value === 'undefined') throw new Error("All fields are compulsory for SDK initialization");
    }
    this.merchantID = merchantID;
    this.successURL = successURL;
    this.failureURL = failureURL;
    this.notifyURL = notifyURL;
    this.signatoryJID = signatoryJID;
    this.signatureClient = new SignatureClient(publicKey, privateKey, zetaPublicKey);
};

function requestOTPPayload(mobileNumber, permissions) {
    return {
        phoneNumber: CommonUtil.getZetaMobileNumberUrl(mobileNumber),
        merchantID: this.merchantID,
        permissions: permissions.join(",")
    };
}

function verifyOtpPayload(requestID, OTP) {
    return {
        requestID: requestID,
        otp: OTP,
        merchantID: this.merchantID
    }
}

function getHasSufficientBalancePayload(mobileNumber, amountInPaise, userToken) {
    let formattedUserIdentifier = CommonUtil.getZetaMobileNumberUrl(mobileNumber);
    let hasSufficientBalanceRequest = {
        userIdentifier: formattedUserIdentifier,
        transactionValue: {
            currency: "INR",
            amount: amountInPaise
        },
        merchantID: this.merchantID,
        userToken: userToken,
        time: new Date().getTime(),
    };

    hasSufficientBalanceRequest.headers = {
        signatoryJID: this.signatoryJID,
        signature: this.signatureClient.generate(hasSufficientBalanceRequest)
    };
    return hasSufficientBalanceRequest;
}

function getPaymentStatusPayload(paymentRequestID) {
    let paymentStatusRequest = {
        paymentRequestID: paymentRequestID,
        merchantID: this.merchantID
    };
    let signature = this.signatureClient.generate(paymentStatusRequest);
    paymentStatusRequest.headers = {
        signatoryJID: this.signatoryJID,
        signature: signature
    };
    return paymentStatusRequest;
}

function getRequestPaymentPayload(paymentRequestID, mobileNumber, amountInPaise, remarks, userToken) {
    let dueBy = new Date().getTime() + (30 * 60 * 1000);
    let attributes = {
        "pwz.remarks": remarks,
        "pwz.autoDebit": true
    };
    let paymentPayload = {
        paymentRequestID: paymentRequestID,
        merchantID: this.merchantID,
        dueBy: dueBy,
        txnAmount: {
            amount: amountInPaise,
            currency: "INR"
        },
        attributes: attributes
    };
    let headers = {
        "pwz.merchant.successUrl": this.successURL,
        "pwz.merchant.failureUrl": this.failureURL,
        "pwz.merchant.notifyUrl": this.notifyURL,
        "pwz.merchant.sdk.version": "1.0",
        "pwz.merchant.sdk.name": "pwz-node",
        "signatoryJID": this.signatoryJID
    };
    if (userToken) {
        headers["pwz.userToken"] = userToken;
    }
    if (mobileNumber) {
        headers["pwz.userIdentificationVector"] = CommonUtil.getZetaMobileNumberUrl(mobileNumber);

    }
    headers["signature"] = this.signatureClient.generate(paymentPayload);
    paymentPayload["headers"] = headers;
    return paymentPayload;
}

function requestOTP(mobileNUmber, permissions) {
    let payload = this.requestOTPPayload(mobileNUmber, permissions);
    return CommonUtil.makeAPICall(payload, config.PWZ_URL.requestOTP);
}

function validateZetaSignature(paymentResponse) {
    // Util.getSignableReresentation(paymentResponse);
    // this.signatureClient.validateResponse
}

function hasBalance(mobileNumber, amountInPaise, userToken) {
    let payload = this.getHasSufficientBalancePayload(mobileNumber, amountInPaise, userToken);
    return CommonUtil.makeAPICall(payload, config.PWZ_URL.hasBalance);
}

function requestPayment(requestID, mobileNumber, amountInPaise, remarks, userToken) {
    let payload = this.getRequestPaymentPayload(requestID, mobileNumber, amountInPaise, remarks, userToken);
    return CommonUtil.makeAPICall(payload, config.PWZ_URL.requestPayment);
}

function verifyOTP(requestID, OTP) {
    let payload = this.verifyOtpPayload(requestID, OTP);
    return CommonUtil.makeAPICall(payload, config.PWZ_URL.verifyOTP);
}

function getPaymentStatus(paymentRequestID) {
    let payload = this.getPaymentStatusPayload(paymentRequestID);
    return CommonUtil.makeAPICall(payload, config.PWZ_URL.paymentStatus);
}


PwZSDK.prototype.getRequestPaymentPayload = getRequestPaymentPayload;
PwZSDK.prototype.verifyOtpPayload = verifyOtpPayload;
PwZSDK.prototype.requestOTPPayload = requestOTPPayload;
PwZSDK.prototype.getPaymentStatusPayload = getPaymentStatusPayload;
PwZSDK.prototype.getHasSufficientBalancePayload = getHasSufficientBalancePayload;
PwZSDK.prototype.requestOTP = requestOTP;
PwZSDK.prototype.verifyOTP = verifyOTP;
PwZSDK.prototype.hasBalance = hasBalance;
PwZSDK.prototype.requestPayment = requestPayment;
PwZSDK.prototype.getPaymentStatus = getPaymentStatus;

module.exports = PwZSDK;